@@ -4,10 +4,11 @@
 # Copyright, 2017-2024, by Samuel Williams.
 # Copyright, 2017, by Kent Gruber.
 # Copyright, 2022, by Shannon Skipper.
+# Copyright, 2025, by Shopify Inc.
 
-require 'fiber/annotation'
+require "fiber/annotation"
 
-require_relative 'list'
+require_relative "list"
 
 module Async
 	# A list of children tasks.
@@ -180,6 +181,7 @@ module Async
 			"\#<#{self.description}>"
 		end
 		
+		# @returns [String] A description of the node.
 		alias inspect to_s
 		
 		# Change the parent of this node.
