@@ -1,9 +1,9 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2021-2022, by Samuel Williams.
+# Copyright, 2021-2025, by Samuel Williams.
 
-require_relative 'condition'
+require_relative "condition"
 
 module Async
 	# A synchronization primitive that allows one task to wait for another task to resolve a value.
@@ -31,6 +31,11 @@ module Async
 			condition.signal(value)
 		end
 		
+		# Alias for {#resolve}.
+		def value=(value)
+			self.resolve(value)
+		end
+		
 		# Whether the value has been resolved.
 		#
 		# @returns [Boolean] Whether the value has been resolved.
@@ -41,14 +46,14 @@ module Async
 		# Wait for the value to be resolved.
 		#
 		# @returns [Object] The resolved value.
-		def value
+		def wait
 			@condition&.wait
 			return @value
 		end
 		
-		# Alias for {#value}.
-		def wait
-			self.value
+		# Alias for {#wait}.
+		def value
+			self.wait
 		end
 	end
 end
